#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from gpiozero import LED, Button
from time import sleep
import array

clock = LED(24)
mosi = LED(23)
miso = Button(27)
select = LED(22)
hold_reset = LED(25)
write_protect = LED(17)

PAUSE = 0.01

# Winbond commands
READ_JEDEC_ID = 0x9f
READ_STATUS_REGISTER_BYTES = array.array('I', [0x05, 0x35, 0x15])
WRITE_STATUS_REGISTER_BYTES = array.array('I', [0x01, 0x31, 0x11])
WRITE_ENABLE = 0x06
WRITE_DISABLE = 0x04

def prepare_SPI_lines():
  print("preparing the SPI lines")

  select.on()
  clock.off()
  mosi.off();

  # hold is low active
  hold_reset.on();
  # write protect is low active
  write_protect.on();

def select_chip():
  # low active
  select.off()

def unselect_chip():
  select.on()

# send the byte bit by bit
def write_byte(single_byte):
  print("Writing byte: " + '0x{:02X}'.format(single_byte))

  for i in range(7, -1, -1):
    mosi.value = (single_byte >> i) & 1
    clock.on()
    sleep(PAUSE)
    clock.off()
    sleep(PAUSE)

# read some bytes
def read_bytes(number_of_bytes):
  return_value = 0
  for i in range(number_of_bytes * 8):
    clock.on()
    # read on rising edge, button value is inverted
    return_value = (return_value << 1) | (miso.value ^ 1)
    sleep(PAUSE)
    clock.off()
    sleep(PAUSE)
  return return_value

def read_jedec_id():
  print()
  select_chip()

  write_byte(READ_JEDEC_ID)
  # receive the 3 byte Jedec ID
  jedec_id = read_bytes(3)

  unselect_chip()

  print("JEDEC ID: " + '0x{:02X}'.format(jedec_id))

##### main #####

if __name__ == "__main__":
  prepare_SPI_lines()
  read_jedec_id()
